import React, { useEffect, useRef, useState } from 'react'
import { Box } from '@mui/material'
import { useTheme, type Theme } from '@mui/material/styles'
import { useRemoteUsers, usePublish, useRTCClient, useClientEvent } from 'agora-rtc-react'
import SymmetricGrid from '../SymmetricGrid/SymmetricGrid'
import PinnedGrid from '../PinnedGrid/PinnedGrid'
import type {
  IAgoraRTCRemoteUser,
  ICameraVideoTrack,
  IMicrophoneAudioTrack,
} from 'agora-rtc-sdk-ng'
import ControlCenter from '../ControlCenter/ControlCenter'

interface Props {
  setJoined: (x: boolean) => void
  localCameraTrack: ICameraVideoTrack | null
  localMicrophoneTrack: IMicrophoneAudioTrack | null
}

const MeetingLayout = ({ setJoined, localCameraTrack, localMicrophoneTrack }: Props) => {
  const theme: Theme = useTheme()

  const agoraEngine = useRTCClient()
  usePublish([localMicrophoneTrack, localCameraTrack])
  const remoteUsers: IAgoraRTCRemoteUser[] = useRemoteUsers()

  const gridContainerRef = useRef<HTMLDivElement>(null)
  const [gridContainerHeight, setGridContainerHeight] = useState(0)

  const [totalUsers, setTotalUsers] = useState(0)
  const [isPinned, setIsPinned] = useState(false)
  const [pinnedUser, setPinnedUser] = useState<IAgoraRTCRemoteUser | null>(null)
  const [isCaptionEnable, setIsCaptionEnable] = useState(false)

  useEffect(() => {
    setTotalUsers(remoteUsers.length + 1)
  }, [remoteUsers])

  useEffect(() => {
    if (gridContainerRef.current) {
      setGridContainerHeight(gridContainerRef.current.offsetHeight)
    }
  }, [gridContainerRef?.current?.offsetHeight, isCaptionEnable])

  const handleUserLeft = (user: IAgoraRTCRemoteUser) => {
    if (user.uid === pinnedUser?.uid) {
      setIsPinned(false)
      setPinnedUser(null)
    }
  }

  useClientEvent(agoraEngine, 'user-left', handleUserLeft)

  return (
    <Box
      sx={{
        width: '100%',
        height: '100%',
        backgroundColor: theme.palette.background.default,
        padding: '30px',
        display: 'flex',
        flexDirection: 'column',
        justifyContent: 'space-between',
      }}
    >
      <Box
        sx={{
          display: 'flex',
          flexDirection: 'row',
          width: '100%',
          height: isCaptionEnable ? '70%' : '85%',
        }}
        ref={gridContainerRef}
      >
        {isPinned && pinnedUser ? (
          <PinnedGrid
            totalUsers={totalUsers}
            gridContainerHeight={gridContainerHeight}
            setIsPinned={setIsPinned}
            pinnedUser={pinnedUser}
            setPinnedUser={setPinnedUser}
            localCameraTrack={localCameraTrack}
          />
        ) : (
          <SymmetricGrid
            totalUsers={totalUsers}
            gridContainerHeight={gridContainerHeight}
            isPinned={isPinned}
            setIsPinned={setIsPinned}
            setPinnedUser={setPinnedUser}
            localCameraTrack={localCameraTrack}
          />
        )}
      </Box>
      {isCaptionEnable && (
        <Box
          sx={{
            height: '10%',
            padding: '20px 40px',
            backgroundColor: '#363739',
            borderRadius: '20px',
            marginRight: '20px',
          }}
        ></Box>
      )}
      <Box sx={{ height: '10%', width: '100%', display: 'flex', marginBottom: '40px' }}>
        <ControlCenter
          setJoined={setJoined}
          localCameraTrack={localCameraTrack}
          localMicrophoneTrack={localMicrophoneTrack}
          isCaptionEnable={isCaptionEnable}
          setIsCaptionEnable={setIsCaptionEnable}
        />
      </Box>
    </Box>
  )
}

export default MeetingLayout
