import React, { useEffect } from 'react'
import type { IAgoraRTCRemoteUser, ICameraVideoTrack } from 'agora-rtc-sdk-ng'
import { Box, Button, IconButton, Popover, Stack } from '@mui/material'
import useMediaQuery from '@mui/material/useMediaQuery'
import { RemoteUser, useRemoteUsers } from 'agora-rtc-react'
import { useTheme, type Theme } from '@mui/material/styles'
import UserCard from '../UserCard/UserCard'
import MoreVertIcon from '@mui/icons-material/MoreVert'

interface Props {
  totalUsers: number
  gridContainerHeight: number
  setIsPinned: (value: boolean) => void
  pinnedUser: IAgoraRTCRemoteUser
  setPinnedUser: (value: IAgoraRTCRemoteUser | null) => void
  localCameraTrack: ICameraVideoTrack | null
}

const PinnedGrid = ({
  totalUsers,
  gridContainerHeight,
  setIsPinned,
  pinnedUser,
  setPinnedUser,
  localCameraTrack,
}: Props) => {
  const theme: Theme = useTheme()
  const isSm = useMediaQuery(theme.breakpoints.up('sm'))

  const remoteUsers = useRemoteUsers()

  const [anchorEl, setAnchorEl] = React.useState<HTMLButtonElement | null>(null)

  const [itemHeight, setItemHeight] = React.useState((gridContainerHeight - 16 * 3) / 4)
  const open = Boolean(anchorEl)
  const menuId = open ? 'simple-popover' : undefined

  const handleClickMenu = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget)
  }

  const handleCloseMenu = () => {
    setAnchorEl(null)
  }

  const handleUnpinUser = () => {
    setIsPinned(false)
    setPinnedUser(null)
    setAnchorEl(null)
  }

  useEffect(() => {
    setItemHeight((gridContainerHeight - 16 * 2) / 3)
  }, [gridContainerHeight])

  return (
    <Box
      sx={{
        display: 'flex',
        width: '100%',
        flexDirection: 'row',
        justifyContent: 'space-between',
        height: '100%',
      }}
    >
      <Box
        sx={{
          width: isSm ? '65%' : '100%',
          backgroundColor: '#262625',
          height: '100%',
          borderRadius: '15px',
          marginX: '10px',
          position: 'relative',
        }}
      >
        <RemoteUser
          user={pinnedUser}
          playVideo={true}
          playAudio={true}
          style={{ width: '100%', height: '100%', borderRadius: '15px' }}
        />
        <IconButton
          aria-lable='Menu'
          sx={{
            position: 'absolute',
            top: '5%',
            right: '5%',
            borderRadius: '50%',
            padding: '10px',
            zIndex: 100,
            pointerEvents: 'auto',
            '&:hover': { backgroundColor: '#363739', opacity: 0.8 },
          }}
          onClick={handleClickMenu}
        >
          <MoreVertIcon />
        </IconButton>
        <Popover
          id={menuId}
          open={open}
          anchorEl={anchorEl}
          onClose={handleCloseMenu}
          anchorOrigin={{
            vertical: 'bottom',
            horizontal: 'left',
          }}
        >
          <Box
            sx={{
              padding: '5px 20px',
              display: 'flex',
              flexDirection: 'columnÍ',
              backgroundColor: '#363739',
              width: '150px',
            }}
          >
            <Button
              sx={{
                color: theme.palette.common.white,
                width: '100%',
                textAlign: 'left',
                justifyContent: 'start',
                textTransform: 'none',
              }}
              onClick={handleUnpinUser}
            >
              Unpin User
            </Button>
          </Box>
        </Popover>
      </Box>
      {isSm && (
        <Box
          sx={{
            width: '30%',
            display: 'flex',
            flexDirection: 'column',
            alignItems: 'center',
            justifyContent: 'center',
            height: '100%',
          }}
        >
          <Stack spacing={4} sx={{ width: '100%', height: 'fit-content' }}>
            <UserCard
              type={'local'}
              itemHeight={itemHeight}
              setIsPinned={setIsPinned}
              setPinnedUser={setPinnedUser}
              localCameraTrack={localCameraTrack}
            />
            {remoteUsers.length > 1 &&
              remoteUsers
                .filter((user) => user.uid !== pinnedUser.uid)
                .map((user) => (
                  <UserCard
                    key={user.uid}
                    type={'remote'}
                    remoteUser={user}
                    itemHeight={itemHeight}
                    setIsPinned={setIsPinned}
                    setPinnedUser={setPinnedUser}
                  />
                ))}
          </Stack>
        </Box>
      )}
    </Box>
  )
}

export default PinnedGrid
