import { Box, Button, IconButton, Popover } from '@mui/material'
import MoreVertIcon from '@mui/icons-material/MoreVert'
import { LocalVideoTrack, RemoteUser } from 'agora-rtc-react'
import type { IAgoraRTCRemoteUser, ICameraVideoTrack } from 'agora-rtc-sdk-ng'
import { useTheme, type Theme } from '@mui/material/styles'
import React, { useEffect } from 'react'

interface Props {
  type: 'remote' | 'local'
  itemHeight: number
  setIsPinned: (value: boolean) => void
  setPinnedUser: (value: IAgoraRTCRemoteUser | null) => void
  remoteUser?: IAgoraRTCRemoteUser
  localCameraTrack?: ICameraVideoTrack | null
}

const UserCard = ({
  type,
  remoteUser,
  itemHeight,
  setIsPinned,
  setPinnedUser,
  localCameraTrack,
}: Props) => {
  const theme: Theme = useTheme()

  const [anchorEl, setAnchorEl] = React.useState<HTMLButtonElement | null>(null)

  const [isLocal, setIsLocal] = React.useState<boolean>(type === 'local')

  const open = Boolean(anchorEl)
  const menuId = open ? 'simple-popover' : undefined

  useEffect(() => {
    setIsLocal(type === 'local')
  }, [type])

  const handleClickMenu = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget)
  }

  const handleCloseMenu = () => {
    setAnchorEl(null)
  }

  const handlePinUser = () => {
    setIsPinned(true)
    setPinnedUser(remoteUser ?? null)
    setAnchorEl(null)
  }

  return (
    <Box
      sx={{
        backgroundColor: '#262625',
        height: `${itemHeight}px`,
        borderRadius: '15px',
        width: '100%',
        position: 'relative',
        overflow: 'hidden',
      }}
    >
      {isLocal ? (
        <LocalVideoTrack
          track={localCameraTrack}
          play={true}
          style={{ width: '100%', height: '100%', borderRadius: '20%' }}
        />
      ) : (
        <RemoteUser
          user={remoteUser}
          playVideo={true}
          playAudio={true}
          style={{ width: '100%', height: '100%', borderRadius: '15px' }}
        />
      )}
      {!isLocal && (
        <>
          <IconButton
            aria-lable='Menu'
            sx={{
              position: 'absolute',
              top: '5%',
              right: '5%',
              borderRadius: '50%',
              padding: '10px',
              zIndex: 100,
              pointerEvents: 'auto',
              '&:hover': { backgroundColor: '#363739', opacity: 0.8 },
            }}
            onClick={handleClickMenu}
          >
            <MoreVertIcon />
          </IconButton>
          <Popover
            id={menuId}
            open={open}
            anchorEl={anchorEl}
            onClose={handleCloseMenu}
            anchorOrigin={{
              vertical: 'bottom',
              horizontal: 'left',
            }}
          >
            <Box
              sx={{
                padding: '5px 20px',
                display: 'flex',
                flexDirection: 'columnÍ',
                backgroundColor: '#363739',
                width: '150px',
              }}
            >
              <Button
                sx={{
                  color: theme.palette.common.white,
                  width: '100%',
                  textAlign: 'left',
                  justifyContent: 'start',
                  textTransform: 'none',
                }}
                onClick={handlePinUser}
              >
                Pin User
              </Button>
            </Box>
          </Popover>
        </>
      )}
    </Box>
  )
}

export default UserCard
